!
!---------------------------------------------------------------------
!
!Purpose:
! to track a general 2-D geometry using the SALT tracking procedure.
!
!Copyright:
! Copyright (C) 2014 Ecole Polytechnique de Montreal.
!
!Author(s):
! A. Hebert
!
!Parameters: input
! IPRINT  print level.
! LINMAX  maximum number of segments in a track.
! ORITRK  a point on the track (origin).
! DIRTRK  the track direction (direction cosines).
! ITDIR   phi value index
! IBLIN   track line starting point
!
!Parameters: output
! IELIN   track line ending point.
! NUMERO  region/surface identification number for segment.
! DLENGT  spatial location of each line segment.
!
!---------------------------------------------------------------------
!
SUBROUTINE SALLCA(IPRINT,LINMAX,ORITRK,DIRTRK,ITDIR,IBLIN,IELIN,NUMERO,DLENGT)
  USE GANLIB
  USE PRECISION_AND_KINDS, ONLY : PDB,INFINITY
  USE SAL_GEOMETRY_MOD,    ONLY : GG
  USE SAL_TISO_MOD,        ONLY : SAL231
  USE SAL_TRAJECTORY_MOD
  USE SAL_TRACKING_TYPES,  ONLY : ITRAC2,RTRAC2,NNN,EX0,EY0,AX,AY,IERR,CNT,CNT0, &
                                  DELX,DD0
  !----
  !  Subroutine arguments
  !----
  INTEGER :: IPRINT,LINMAX,ITDIR,IBLIN,IELIN,NUMERO(LINMAX)
  REAL(PDB) :: ORITRK(2),DIRTRK(2),DLENGT(LINMAX)
  !----
  !  Local variables
  !----
  REAL(PDB) :: ANGLE
  !----
  !  Process track
  !----
  EX0=DIRTRK(1)
  EY0=DIRTRK(2)
  AX=ORITRK(1)
  AY=ORITRK(2)
  ANGLE=DACOS(DIRTRK(1))
  IERR=0
  CNT0=0
  DD0=-INFINITY
  CNT=CNT0+NNN
  CALL SALTRA(GG%NPERIM_MAC2,GG%PERIM_MAC2,GG%ISURF2_ELEM,GG%IPAR,GG%RPAR,&
              GG%PPERIM_NODE,GG%PERIM_NODE,ITDIR)
    
  ! a trajectory has been completed: store angle order NBER, total weight and WR
  II0=CNT0+1
  IF(IPRINT>3) THEN
    CALL SAL231(RTRAC2(II0:),ITRAC2(II0:),DELX,EX0,EY0,ANGLE)
  ENDIF
  NTRACK=ITRAC2(II0)
  IF(IERR==0) THEN
    IELIN=IBLIN
  ELSE
    IELIN=IBLIN+NTRACK+1
    IF(IELIN > LINMAX) CALL XABORT('SALLCA: LINMAX overflow.')
    DLENGT(IBLIN)=1._PDB
    DLENGT(IELIN)=1._PDB
    NUMERO(IBLIN)=-ITRAC2(II0+5-1)
    NUMERO(IELIN)=-ITRAC2(II0+6-1)
    DLENGT(IBLIN+1:IELIN-1)=RTRAC2(II0+NNN:II0+NNN+NTRACK-1)
    NUMERO(IBLIN+1:IELIN-1)=GG%NUM_MERGE(ITRAC2(II0+NNN:II0+NNN+NTRACK-1))
  ENDIF
  RETURN
END SUBROUTINE SALLCA
