*DECK SPHMAC
      SUBROUTINE SPHMAC(IPMACR,IPRINT,NMERGE,NGCOND,ISCAT,NWGTH,NIFISS,
     1 ILEAKS,VOLMER,FLXMER,SUNMER,SIGT,SIGW,DIFF,ZLEAK)
*
*-----------------------------------------------------------------------
*
*Purpose:
* recovery of the reference merged/condensed set of cross sections
* to be used by an SPH homogenization algorithm.
*
*Copyright:
* Copyright (C) 2011 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPMACR  pointer to the condensed macrolib (L_MACROLIB signature).
* IPRINT  print flag (equal to 0 for no print).
* NMERGE  number of merged regions.
* NGCOND  number of condensed groups.
* NMERGE  number of mixtures.
* ISCAT   scattering anisotropy in the reference set of cross sections
*         (=1: isotropic in LAB; =2: linearly-anisotropic in LAB).
* NWGTH   type of weighting for P1 cross section information:
*         = 0 P0; = 1 P1.
* NIFISS  number of fissile isotopes.
* ILEAKS  type of leakage calculation =0: no leakage; =1: homogeneous
*         leakage (Diffon); =2: isotropic streaming (Ecco);
*         =3: anisotropic streaming (Tibere).
*
*Parameters: output
* VOLMER  merged volumes.
* FLXMER  merged/condensed averaged fluxes.
* SUNMER  merged/condensed production (fission + scattering) cross
*         sections. The third dimension is for secondary neutrons.
* SIGT    merged/condensed total cross sections.
* SIGW    merged/condensed within-group scattering cross sections.
* DIFF    merged/condensed diffusion coefficients.
* ZLEAK   merged/condensed leakage rates.
*
*-----------------------------------------------------------------------
*
      USE GANLIB
*----
*  SUBROUTINE ARGUMENTS
*----
      TYPE(C_PTR) IPMACR
      INTEGER IPRINT,NMERGE,NGCOND,ISCAT,NWGTH,NIFISS,ILEAKS
      REAL VOLMER(NMERGE),FLXMER(NMERGE,NGCOND,NWGTH+1),
     1 SUNMER(NMERGE,NGCOND,NGCOND,ISCAT),
     2 SIGT(NMERGE,NGCOND,NWGTH+1),SIGW(NMERGE,NGCOND,ISCAT),
     3 DIFF(NMERGE,NGCOND),ZLEAK(NMERGE,NGCOND,NWGTH+1)
*----
*  LOCAL VARIABLES
*----
      PARAMETER(NSTATE=40)
      INTEGER ISTATE(NSTATE)
      DOUBLE PRECISION ZNUMER,ZLEAKA,ZDENUM
      CHARACTER HSIGN*12,SUFF*2
      TYPE(C_PTR) JPMACR,KPMACR
      INTEGER, ALLOCATABLE, DIMENSION(:) :: NJJ,IJJ,IPOS
      REAL, ALLOCATABLE, DIMENSION(:) :: SIGMA,XSCAT,DIFHOM,SIG1
      REAL, ALLOCATABLE, DIMENSION(:,:,:) :: PRODUC
*----
*  SCRATCH STORAGE ALLOCATION
*----
      ALLOCATE(NJJ(NMERGE),IJJ(NMERGE),IPOS(NMERGE))
      ALLOCATE(PRODUC(NMERGE,NGCOND,NIFISS),SIGMA(NMERGE*NIFISS),
     1 XSCAT(NMERGE*NGCOND),DIFHOM(NGCOND))
*----
*  RECOVER MACROLIB INFORMATION
*----
      CALL LCMGTC(IPMACR,'SIGNATURE',12,1,HSIGN)
      IF(HSIGN.NE.'L_MACROLIB') CALL XABORT('SPHMAC: MACROLIB EXPECTED')
      CALL LCMGET(IPMACR,'STATE-VECTOR',ISTATE)
      IF(ISTATE(1).NE.NGCOND) CALL XABORT('SPHMAC: INVALID NGCOND')
      IF(ISTATE(2).NE.NMERGE) CALL XABORT('SPHMAC: INVALID NMERGE')
      NL=ISTATE(3)
      IF(ISTATE(4).NE.NIFISS) CALL XABORT('SPHMAC: INVALID NIFISS')
      ITRANC=ISTATE(6)
      ILEAK=ISTATE(9)
      IF(NWGTH.GT.ISTATE(10)) CALL XABORT('SPHMAC: INVALID NWGTH')
*----
*  SET OUTPUT INFORMATION TO ZERO
*----
      CALL XDRSET(PRODUC,NMERGE*NGCOND*NIFISS,0.0)
      CALL XDRSET(FLXMER,NMERGE*NGCOND*(NWGTH+1),0.0)
      CALL XDRSET(SIGT,NMERGE*NGCOND*(NWGTH+1),0.0)
      CALL XDRSET(SIGW,NMERGE*NGCOND*ISCAT,0.0)
      CALL XDRSET(SUNMER,NMERGE*NGCOND*NGCOND*ISCAT,0.0)
      CALL XDRSET(ZLEAK,NMERGE*NGCOND*(NWGTH+1),0.0)
*----
*  RECOVER FLUX AND COMPUTE THE FISSION RATE INFORMATION
*----
      ZNUMER=0.0D0
      ZLEAKA=0.0D0
      ZDENUM=0.0D0
      CALL LCMGET(IPMACR,'VOLUME',VOLMER)
      JPMACR=LCMGID(IPMACR,'GROUP')
      DO 40 IGR=1,NGCOND
      KPMACR=LCMGIL(JPMACR,IGR)
      CALL LCMLEN(KPMACR,'FLUX-INTG',ILCMLN,ITYLCM)
      IF(ILCMLN.EQ.0) CALL XABORT('SPHMAC: MISSING FLUX-INTG INFO')
      CALL LCMGET(KPMACR,'FLUX-INTG',FLXMER(1,IGR,1))
      IF(NWGTH.GT.0) THEN
         CALL LCMLEN(KPMACR,'FLUX-INTG-P1',ILCMLN,ITYLCM)
         IF(ILCMLN.GT.0) THEN
            CALL LCMGET(KPMACR,'FLUX-INTG-P1',FLXMER(1,IGR,2))
         ELSE
            DO 20 IBM=1,NMERGE
  20        FLXMER(IBM,IGR,2)=FLXMER(IBM,IGR,1)
         ENDIF
      ENDIF
      CALL LCMLEN(KPMACR,'NUSIGF',ILCMLN,ITYLCM)
      IF(ILCMLN.GT.0) THEN
         CALL LCMGET(KPMACR,'NUSIGF',SIGMA(1))
         DO 30 IFIS=1,NIFISS
         DO 30 IBM=1,NMERGE
         SS=FLXMER(IBM,IGR,1)*SIGMA((IFIS-1)*NMERGE+IBM)
         PRODUC(IBM,IGR,IFIS)=PRODUC(IBM,IGR,IFIS)+SS
  30     ZNUMER=ZNUMER+SS
      ENDIF
  40  CONTINUE
*----
*  RECOVER EIGENVALUES
*----
      CALL LCMLEN(IPMACR,'K-EFFECTIVE',ILCMLN,ITYLCM)
      IF(ILCMLN.EQ.1) THEN
         CALL LCMGET(IPMACR,'K-EFFECTIVE',EIGENK)
      ELSE
         EIGENK=1.0
      ENDIF
      IF(IPRINT.GT.5) WRITE(6,'(/16H SPHMAC: EIGENK=,1P,E12.4)') EIGENK
      CALL LCMLEN(IPMACR,'B2  B1HOM',ILCMLN,ITYLCM)
      IF(ILCMLN.EQ.1) THEN
         CALL LCMGET(IPMACR,'B2  B1HOM',B2)
      ELSE
         B2=0.0
      ENDIF
      IF(IPRINT.GT.5) WRITE(6,'(/12H SPHMAC: B2=,1P,E12.4)') B2
*----
*  RECOVER MERGED/CONDENSED CROSS SECTIONS
*----
      DO 170 IGR=1,NGCOND
      KPMACR=LCMGIL(JPMACR,IGR)
      CALL LCMLEN(KPMACR,'NTOT0',ILCMLN,ITYLCM)
      IF(ILCMLN.GT.0) THEN
         CALL LCMGET(KPMACR,'NTOT0',SIGMA(1))
      ELSE
         CALL XABORT('SPHMAC: MISSING NTOT0 INFO')
      ENDIF
      DO 45 IBM=1,NMERGE
   45 ZDENUM=ZDENUM+SIGMA(IBM)*FLXMER(IBM,IGR,1)
      IF(ITRANC.NE.0) THEN
*        TRANSPORT CORRECTION.
         ALLOCATE(SIG1(NMERGE))
         CALL LCMGET(KPMACR,'TRANC',SIG1)
         DO 50 IBM=1,NMERGE
   50    SIGMA(IBM)=SIGMA(IBM)-SIG1(IBM)
         DEALLOCATE(SIG1)
      ENDIF
      IF(ILEAKS.EQ.1) THEN
         CALL LCMLEN(KPMACR,'DIFF',ILCMLN,ITYLCM)
         IF(ILCMLN.GT.0) THEN
            CALL LCMGET(KPMACR,'DIFF',DIFF(1,IGR))
         ELSE
            CALL LCMGET(IPMACR,'DIFHOMB1HOM',DIFHOM)
            DO 55 I=1,NMERGE
   55       DIFF(I,IGR)=DIFHOM(IGR)
         ENDIF
      ENDIF
      DO 60 IBM=1,NMERGE
      IF((ILEAKS.EQ.1).AND.(NWGTH.EQ.0)) THEN
         ZLEAK(IBM,IGR,1)=DIFF(IBM,IGR)*B2*FLXMER(IBM,IGR,1)
      ELSE IF((ILEAKS.EQ.1).AND.(NWGTH.EQ.1)) THEN
         ZLEAK(IBM,IGR,1)=B2*FLXMER(IBM,IGR,2)
      ELSE IF(ILEAKS.GT.1) THEN
         CALL XABORT('SPHMAC: LEAKAGE MODEL NOT IMPLEMENTED')
      ELSE
         ZLEAK(IBM,IGR,1)=0.0
      ENDIF
      SIGT(IBM,IGR,1)=SIGMA(IBM)
      ZLEAKA=ZLEAKA+ZLEAK(IBM,IGR,1)
   60 CONTINUE
*
      CALL LCMLEN(KPMACR,'CHI',ILCMLN,ITYLCM)
      IF(ILCMLN.GT.0) THEN
         DO 80 IFIS=1,NIFISS
         CALL LCMGET(KPMACR,'CHI',SIGMA(1))
         DO 80 IBM=1,NMERGE
         DO 70 JGR=1,NGCOND
         SS=SIGMA((IFIS-1)*NMERGE+IBM)*PRODUC(IBM,JGR,IFIS)/EIGENK
         SUNMER(IBM,JGR,IGR,1)=SUNMER(IBM,JGR,IGR,1)+SS
   70    CONTINUE
   80    CONTINUE
      ENDIF
      IF(NWGTH.EQ.1) THEN
         CALL LCMLEN(KPMACR,'NTOT1',ILCMLN,ITYLCM)
         IF(ILCMLN.GT.0) THEN
            CALL LCMGET(KPMACR,'NTOT1',SIGMA(1))
         ELSE
            CALL XABORT('SPHMAC: MISSING NTOT1 INFO')
         ENDIF
         DO 90 IBM=1,NMERGE
         IF((ILEAKS.EQ.1).AND.(NWGTH.EQ.1)) THEN
            ZLEAK(IBM,IGR,2)=-FLXMER(IBM,IGR,1)/3.0
         ELSE IF(ILEAKS.GT.1) THEN
            CALL XABORT('SPHMAC: LEAKAGE MODEL NOT IMPLEMENTED')
         ELSE
            ZLEAK(IBM,IGR,2)=0.0
         ENDIF
         SIGT(IBM,IGR,2)=SIGMA(IBM)
   90    CONTINUE
      ENDIF
*----
*  PROCESS SCATTERING INFORMATION
*----
      DO 170 INL=1,ISCAT
      INFL=MIN(INL,NWGTH+1)
      WRITE(SUFF,'(I2.2)') INL-1
      CALL LCMLEN(KPMACR,'SIGW'//SUFF,ILCMLN,ITYLCM)
      IF(ILCMLN.GT.0) THEN
         CALL LCMGET(KPMACR,'SIGW'//SUFF,SIGMA(1))
      ELSE
         CALL XDRSET(SIGMA,NMERGE,0.0)
      ENDIF
      IF((ITRANC.NE.0).AND.(INL.EQ.1)) THEN
*        TRANSPORT CORRECTION.
         ALLOCATE(SIG1(NMERGE))
         CALL LCMGET(KPMACR,'TRANC',SIG1)
         DO 120 IBM=1,NMERGE
  120    SIGMA(IBM)=SIGMA(IBM)-SIG1(IBM)
         DEALLOCATE(SIG1)
      ENDIF
      CALL LCMLEN(KPMACR,'NJJS'//SUFF,ILCMLN,ITYLCM)
      IF(ILCMLN.GT.0) THEN
         CALL LCMGET(KPMACR,'NJJS'//SUFF,NJJ)
         CALL LCMGET(KPMACR,'IJJS'//SUFF,IJJ)
         CALL LCMGET(KPMACR,'IPOS'//SUFF,IPOS)
         CALL LCMGET(KPMACR,'SCAT'//SUFF,XSCAT)
         DO 150 IBM=1,NMERGE
         IPO=IPOS(IBM)
         DO 130 JGR=IJJ(IBM),IJJ(IBM)-NJJ(IBM)+1,-1
         IF(IGR.EQ.JGR) THEN
            SS=SIGMA(IBM)*FLXMER(IBM,JGR,INFL)
         ELSE
            SS=XSCAT(IPO)*FLXMER(IBM,JGR,INFL)
         ENDIF
         IF(IGR.EQ.JGR) SIGW(IBM,IGR,INL)=SIGW(IBM,IGR,INL)+SS
         SUNMER(IBM,JGR,IGR,INL)=SUNMER(IBM,JGR,IGR,INL)+SS
         IF(INL.EQ.1) ZDENUM=ZDENUM-SS
  130    IPO=IPO+1
  150    CONTINUE
      ENDIF
  170 CONTINUE
*
      DO 200 INL=1,ISCAT
      INFL=MIN(INL,NWGTH+1)
      DO 200 IGR=1,NGCOND
      DO 200 IBM=1,NMERGE
      IF(VOLMER(IBM).NE.0.0) THEN
         SIGW(IBM,IGR,INL)=SIGW(IBM,IGR,INL)/FLXMER(IBM,IGR,INFL)
         DO 190 JGR=1,NGCOND
         SUNMER(IBM,JGR,IGR,INL)=SUNMER(IBM,JGR,IGR,INL)
     >                              /FLXMER(IBM,JGR,INFL)
  190   CONTINUE
      ENDIF
  200 CONTINUE
      DO 210 INL=1,NWGTH+1
      DO 210 IGR=1,NGCOND
      DO 210 IBM=1,NMERGE
      IF(VOLMER(IBM).NE.0.0) THEN
         ZLEAK(IBM,IGR,INL)=ZLEAK(IBM,IGR,INL)/FLXMER(IBM,IGR,INL)
         FLXMER(IBM,IGR,INL)=FLXMER(IBM,IGR,INL)/VOLMER(IBM)
      ENDIF
  210 CONTINUE
*----
*  PRINT INFORMATION
*----
      IF(IPRINT.GT.4) THEN
         WRITE(6,240) ZNUMER/ZDENUM,ZNUMER/(ZDENUM+ZLEAKA)
         WRITE(6,250) 'VOLMER',(VOLMER(IKK),IKK=1,NMERGE)
         DO 220 INL=1,NWGTH+1
         WRITE(SUFF,'(I2.2)') INL-1
         WRITE(6,250) 'SIGT'//SUFF,((SIGT(IKK,IGR,INL),IKK=1,NMERGE)
     >   ,IGR=1,NGCOND)
         WRITE(6,250) 'FLXMER'//SUFF,((FLXMER(IKK,IGR,INL),
     >   IKK=1,NMERGE),IGR=1,NGCOND)
         WRITE(6,250) 'ZLEAK'//SUFF,((ZLEAK(IKK,IGR,INL),
     >   IKK=1,NMERGE),IGR=1,NGCOND)
  220    CONTINUE
         DO 230 INL=1,ISCAT
         WRITE(SUFF,'(I2.2)') INL-1
         WRITE(6,250) 'SIGW'//SUFF,((SIGW(IKK,IGR,INL),IKK=1,NMERGE)
     >   ,IGR=1,NGCOND)
         WRITE(6,250) 'SUNMER'//SUFF,(((SUNMER(IKK,IGR,JGR,INL),
     >   IKK=1,NMERGE),IGR=1,NGCOND),JGR=1,NGCOND)
  230    CONTINUE
         WRITE(6,250) 'DIFF',((DIFF(IKK,IGR),IKK=1,NMERGE),IGR=1,NGCOND)
      ENDIF
*----
*  SCRATCH STORAGE DEALLOCATION
*----
      DEALLOCATE(DIFHOM,XSCAT,SIGMA,PRODUC)
      DEALLOCATE(IPOS,IJJ,NJJ)
      RETURN
*
  240 FORMAT(/20H SPHMAC: K-INFINITY=,1P,D13.6/8X,12HK-EFFECTIVE=,D13.6)
  250 FORMAT(/26H SPHMAC: VALUES OF VECTOR ,A,4H ARE/(1X,1P,10E13.5))
      END
