*DECK KINDRV
      SUBROUTINE KINDRV(NEN,KEN,CMOD,NGR,NBM,NBFIS,NDG,NLF,ITY,NEL,
     1 LL4,NUN,NUP,TTF,TTP,DT,IMPH,ICL1,ICL2,NADI,MAXOUT,EPSOUT,MAXINR,
     2 EPSINR,IFL,IPR,INORM,IMPX)
*
*-----------------------------------------------------------------------
*
*Purpose:
* driver to perform the space-time kinetics calculations.
*
*Copyright:
* Copyright (C) 2008 Ecole Polytechnique de Montreal.
*
*Author(s): D. Sekki
*
*Parameters: input
* NEN     number of LCM objects used in the module.
* KEN     addresses of LCM objects: (1) L_KINET; (2) L_MACROLIB;
*         (3) L_TRACK; (4) L_SYSTEM; (5) L_MACROLIB.
* CMOD    name of the assembly door (BIVAC or TRIVAC).
* NGR     number of energy groups.
* NBM     number of material mixtures.
* NBFIS   number of fissile isotopes.
* NDG     number of delayed-neutron groups.
* NLF     number of Legendre orders for fluxes.
* ITY     type of finite elements and tracking.
* NEL     total number of finite elements.
* LL4     number of flux unknowns per energy group.
* NUN     total number of unknowns per energy group.
* NUP     number of precursor unknowns per delayed group.
* TTF     value of theta-parameter for fluxes.
* TTP     value of theta-parameter for precursors.
* DT      current time increment.
* IMPH    management of convergence histogram.
* ICL1    number of free iretations in one cycle of the inverse power
*         method
* ICL2    number of accelerated iterations in one cycle
* NADI    number of inner adi iterations per outer iteration
* MAXOUT  maximum number of outer iterations
* EPSOUT  convergence criteria for the flux
* MAXINR  maximum number of thermal iterations.
* EPSINR  thermal iteration epsilon.
* IFL     temporal integration scheme for fluxes.
* IPR     temporal integration scheme for precursors.
* INORM   type of flux normalization (=0: no normalization; =1: imposed
*         factor; =2: maximum flux; =3 initial power).
* IMPX    printing parameter (=0 for no print).
*
*-----------------------------------------------------------------------
*
      USE GANLIB
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER NEN,NGR,NBM,NBFIS,NDG,NLF,ITY,NEL,LL4,NUN,NUP,IMPH,ICL1,
     1 ICL2,NADI,MAXOUT,MAXINR,IFL,IPR,INORM,IMPX
      TYPE(C_PTR) KEN(NEN)
      REAL TTF,TTP,DT,EPSOUT,EPSINR
      CHARACTER CMOD*12
*----
*  LOCAL VARIABLES
*----
      PARAMETER(IOS=6)
      INTEGER MAT(NEL),IDL(NEL),IDLPC(NEL)
      REAL VOL(NEL),PDC(NDG),PMAX(NDG,NBFIS)
      LOGICAL LNUD,LCHD
      TYPE(C_PTR) IPMAC,IPSYS
      REAL, DIMENSION(:), ALLOCATABLE :: DNF
      REAL, DIMENSION(:,:), ALLOCATABLE :: EVECT,DNS,SRC,PHO,OVR
      REAL, DIMENSION(:,:,:), ALLOCATABLE :: PC,CHI,SGF
      REAL, DIMENSION(:,:,:,:), ALLOCATABLE :: SGD,CHD,SGO
*----
*  SCRATCH STORAGE ALLOCATION
*----
      ALLOCATE(EVECT(NUN,NGR),PC(NUP,NDG,NBFIS),SGD(NBM,NBFIS,NGR,NDG))
*----
*  RECOVER INFORMATION
*----
      CALL KDRCPU(TK1)
      TA1=TK1
      IF(IMPX.GT.0) WRITE(IOS,1001)
      CALL LCMGET(KEN(1),'E-KEFF',EVL)
      CALL LCMGET(KEN(1),'LAMBDA-D',PDC)
      CALL LCMGET(KEN(1),'E-IDLPC',IDLPC)
      CALL LCMGET(KEN(3),'VOLUME',VOL)
      CALL LCMGET(KEN(3),'MATCOD',MAT)
      CALL LCMGET(KEN(3),'KEYFLX',IDL)
*----
*  RECOVER CROSS SECTIONS (BEGINNING-OF-STEP)
*----
      ALLOCATE(DNF(NDG),DNS(NGR,NDG))
      CALL LCMLEN(KEN(1),'BETA-D',LEN,ITYL)
      LNUD=(LEN.EQ.NDG)
      IF(LNUD) CALL LCMGET(KEN(1),'BETA-D',DNF)
      CALL LCMLEN(KEN(1),'CHI-D',LEN,ITYL)
      LCHD=(LEN.EQ.NGR*NDG)
      IF(LCHD) CALL LCMGET(KEN(1),'CHI-D',DNS)
      ALLOCATE(OVR(NBM,NGR),CHI(NBM,NBFIS,NGR),CHD(NBM,NBFIS,NGR,NDG),
     1 SGF(NBM,NBFIS,NGR),SGO(NBM,NBFIS,NGR,NDG))
      IF(NEN.EQ.4) THEN
        IPMAC=KEN(2)
        IPSYS=KEN(4)
      ELSE IF(NEN.EQ.6) THEN
        IPMAC=KEN(5)
        IPSYS=KEN(6)
      ENDIF
      CALL KINXSD(IPMAC,NGR,NBM,NBFIS,NDG,EVL,DT,DNF,DNS,LNUD,LCHD,OVR,
     1 CHI,CHD,SGF,SGO)
*----
*  COMPUTE THE SOURCE TERM
*----
      LL4B=LL4
      IF((ITY.EQ.11).OR.(ITY.EQ.13)) LL4B=LL4*NLF/2
      ALLOCATE(SRC(NGR,NUN),PHO(NGR,NUN))
      CALL LCMGET(KEN(1),'E-PREC',PC)
      CALL LCMGET(KEN(1),'E-VECTOR',PHO)
      CALL KINSRC(KEN(3),IPSYS,CMOD,IMPX,IFL,IPR,NGR,NBM,NBFIS,NDG,ITY,
     1 LL4B,NUN,NUP,PDC,TTF,TTP,DT,OVR,CHI,CHD,SGF,SGO,PHO,PC,SRC)
      DEALLOCATE(PHO)
*----
*  RECOVER CROSS SECTIONS (END-OF-STEP)
*----
      CALL KINXSD(KEN(2),NGR,NBM,NBFIS,NDG,EVL,DT,DNF,DNS,LNUD,LCHD,
     1 OVR,CHI,CHD,SGF,SGD)
      DEALLOCATE(DNS,DNF)
*----
*  RECOVER THE BEGINNING-OF-STEP FLUX
*----
      IF(IMPX.GT.0)THEN
        CALL KDRCPU(TA2)
        WRITE(IOS,1002) TA2-TA1
        WRITE(IOS,1003)
      ENDIF
      ALLOCATE(PHO(NGR,NUN))
      CALL LCMGET(KEN(1),'E-VECTOR',EVECT)
      CALL LCMGET(KEN(1),'E-VECTOR',PHO)
*----
*  COMPUTE THE FLUX SOLUTION
*----
      IF(CMOD.EQ.'BIVAC')THEN
        CALL KINSLB(KEN(3),KEN(4),KEN(1),LL4B,ITY,NUN,NGR,IFL,IPR,
     1  NBM,NBFIS,NDG,ICL1,ICL2,IMPX,IMPH,TITR,EPSOUT,MAXINR,EPSINR,
     2  MAXOUT,PDC,TTF,TTP,DT,OVR,CHI,CHD,SGF,SGD,EVECT,SRC)
      ELSEIF(CMOD.EQ.'TRIVAC')THEN
        CALL KINSLT(KEN(3),KEN(4),KEN(1),LL4B,ITY,NUN,NGR,IFL,IPR,
     1  NBM,NBFIS,NDG,ICL1,ICL2,IMPX,IMPH,TITR,EPSOUT,MAXINR,EPSINR,
     2  NADI,MAXOUT,PDC,TTF,TTP,DT,OVR,CHI,CHD,SGF,SGD,EVECT,SRC)
      ENDIF
      DEALLOCATE(SRC)
      DO 10 IGR=1,NGR
      IF(CMOD.EQ.'BIVAC')THEN
        CALL FLDBIV(KEN(3),NEL,NUP,EVECT(1,IGR),MAT,VOL,IDL)
      ELSEIF(CMOD.EQ.'TRIVAC')THEN
        CALL FLDTRI(KEN(3),NEL,NUP,EVECT(1,IGR),MAT,VOL,IDL)
      ENDIF
   10 CONTINUE
      CALL LCMPUT(KEN(1),'E-VECTOR',NGR*NUN,2,EVECT)
*----
*  COMPUTE THE PRECURSOR SOLUTION
*----
      CALL KINPRC(KEN(3),KEN(4),CMOD,NGR,NBM,NBFIS,NDG,NEL,LL4,NUN,NUP,
     1 MAT,VOL,IDLPC,EVECT,PHO,SGD,SGO,PDC,DT,TTP,PC,IPR,IMPX)
      CALL LCMPUT(KEN(1),'E-PREC',NDG*NUP*NBFIS,2,PC)
*----
*  FIND THE MAXIMUM FLUX VALUE
*----
      FMAX=0.0
      IDMX=0
      DO 20 IGR=1,NGR
      DO 20 IEL=1,NEL
      IND=IDL(IEL)
      IF(IND.EQ.0) GO TO 20
      IF(ABS(EVECT(IND,IGR)).GT.FMAX) THEN
        FMAX=EVECT(IND,IGR)
        IDMX=IEL
        IGMX=IGR
      ENDIF
   20 CONTINUE
      IF(IDMX.EQ.0) CALL XABORT('KINDRV: UNABLE TO SET FMAX.')
      IND=IDLPC(IDMX)
      IF(IND.EQ.0) CALL XABORT('KINDRV: UNABLE TO SET PMAX.')
      DO 30 IFIS=1,NBFIS
      DO 30 IDG=1,NDG
   30 PMAX(IDG,IFIS)=PC(IND,IDG,IFIS)
      IF(IMPX.GT.0) WRITE(IOS,1004) FMAX,IDMX,IGMX
      IF(IMPX.GT.0) THEN
        CALL KDRCPU(TK2)
        WRITE(IOS,1005)TK2-TK1
      ENDIF
      CALL LCMPUT(KEN(1),'CTRL-FLUX',1,2,FMAX)
      CALL LCMPUT(KEN(1),'CTRL-PREC',NDG*NBFIS,2,PMAX)
      CALL LCMPUT(KEN(1),'CTRL-IDL',1,1,IDMX)
      CALL LCMPUT(KEN(1),'CTRL-IGR',1,1,IGMX)
*----
*  COMPUTE REACTOR POWER
*----
      IF(INORM.EQ.3) THEN
        CALL KINPOW(KEN(2),NGR,NBM,NUN,NEL,MAT,VOL,IDL,EVECT,POWTOT)
        CALL LCMPUT(KEN(1),'E-POW',1,2,POWTOT)
        IF(IMPX.GT.0) WRITE(6,*) 'REACTOR POWER (W) =',POWTOT
      ENDIF
*----
*  SCRATCH STORAGE DEALLOCATION
*----
      DEALLOCATE(PHO,SGO,SGF,CHD,CHI,OVR)
      DEALLOCATE(SGD,PC,EVECT)
      RETURN
*
 1001 FORMAT(/1X,'=> ASSEMBLY OF THE SYSTEM MATRICES'/)
 1002 FORMAT(/1X,'TOTAL CPU TIME USED FOR THE ASSEMBLING',
     1 1X,'OF ALL SYSTEM MATRICES =',F6.3/)
 1003 FORMAT(/1X,'=> COMPUTING THE KINETICS SOLUTION'/)
 1004 FORMAT(/1X,'CONTROLLING PARAMETERS:',2X,'MAX-VA',
     1 'L',1X,1PE12.5,3X,'IDL #',I5.5,3X,'IGR #',I2.2/)
 1005 FORMAT(/1X,'TOTAL CPU TIME USED FOR KINETICS CALC',
     1 'ULATIONS =',F10.3//1X,'=> SPACE-TIME',1X,
     2 'KINETICS CALCULATION IS DONE.')
      END
